/*****************************************************************************
 * BF609_SSL_Core0.c
 *
 * Second Stage Loader Example for BF609
 *
 * This application provides a simple two application boot loader.
 * On power up, SW1 and SW2 launch Application 1 or Application 2
 *****************************************************************************/

#include <ccblkfn.h>
#include <bfrom.h>
#include <services/gpio/adi_gpio.h>
#include <stdio.h>
#include "adi_initialize.h"
#include "BF609_SSL_Core0.h"


/* memory for two callbacks */
#define GPIO_MEMORY_SIZE (ADI_GPIO_CALLBACK_MEM_SIZE*2u)

/* Memory Addresses for the Applications to be booted
 * Address locations are in bytes
 * */
#define App1 1048576
#define App2 2097152

static bool bError;

static void CheckResult(ADI_GPIO_RESULT result, char *message);
extern void ConfigSoftSwitches(void);

/*
 * Report error if result code is not success
 */
static void CheckResult(ADI_GPIO_RESULT result, char *message)
{
	if (result != ADI_GPIO_SUCCESS) {
		printf("%s failed\n", message);
		bError = true;
	}
}

/*
 * GPIO event Callback function
 */
static void pinIntCallback(ADI_GPIO_PIN_INTERRUPT ePinInt, uint32_t PinIntData,  void *pCBParam)
{
	switch(ePinInt)
	{
	case ADI_GPIO_PIN_INTERRUPT_0:
		/* push button 1 */
		if (PinIntData & ADI_GPIO_PIN_10)
		{
			/* toggle LED 1 */
			adi_gpio_Toggle(ADI_GPIO_PORT_G, ADI_GPIO_PIN_14);
			adi_gpio_Toggle(ADI_GPIO_PORT_G, ADI_GPIO_PIN_15);
			rom_Boot(App1,0,0,0,0x20210002,0);
		}
		break;
	case ADI_GPIO_PIN_INTERRUPT_3:
		/* push button 2 */
		if (PinIntData & ADI_GPIO_PIN_1)
		{
			/* toggle LED 2 */
			adi_gpio_Toggle(ADI_GPIO_PORT_G, ADI_GPIO_PIN_14);
			adi_gpio_Toggle(ADI_GPIO_PORT_G, ADI_GPIO_PIN_15);
			rom_Boot(App2,0,0,0,0x20210002,0);
		}
		break;
	default:
		break;
	}


}

/*
 * main
 */
int main(void)
{
	/**
		 * Initialize managed drivers and/or services that have been added to
		 * the project.
		 * @return zero on success
		 */
		adi_initComponents();

		/**
		 * The default startup code does not include any functionality to allow
		 * core 0 to enable core 1. A convenient way to enable core 1 is to use the
		 * 'adi_core_1_enable' function.
		 */
		adi_core_1_enable();

		/* Begin adding your custom code here */

	int i;
	uint32_t gpioMaxCallbacks;
	ADI_GPIO_RESULT result;
	static uint8_t gpioMemory[GPIO_MEMORY_SIZE];

    *pREG_PORTB_FER = 0u;   /* bug? should reset to 0 */

    adi_initComponents(); /* auto-generated code */

	/* Software Switch Configuration for the ADSP-BF609 EZ-BOARD */
	ConfigSoftSwitches();

    /* init the GPIO service */
	result = adi_gpio_Init(
			(void*)gpioMemory,
			GPIO_MEMORY_SIZE,
			&gpioMaxCallbacks);
	CheckResult(result, "adi_gpio_Init");

	/*
	 * Setup Push Button 1
	 */

	/* set GPIO input */
	result = adi_gpio_SetDirection(
	    ADI_GPIO_PORT_B,
	    ADI_GPIO_PIN_10,
	    ADI_GPIO_DIRECTION_INPUT);
	CheckResult(result, "adi_gpio_SetDirection");

	/* set edge sense mode (PORT B is connected to Pin Interrupt 0) */
	result = adi_gpio_SetPinIntEdgeSense(
		ADI_GPIO_PIN_INTERRUPT_0,
	    ADI_GPIO_PIN_10,
	    ADI_GPIO_SENSE_RISING_EDGE);
	CheckResult(result, "adi_gpio_SetPinIntEdgeSense");

	/* register pinint 0 callback */
	result = adi_gpio_RegisterCallback(
		ADI_GPIO_PIN_INTERRUPT_0,
	    ADI_GPIO_PIN_10,
	    pinIntCallback,
	    (void*)0);
	CheckResult(result, "adi_gpio_RegisterCallback");

	/* set pin interrupt mask */
	result = adi_gpio_EnablePinInterruptMask(
		ADI_GPIO_PIN_INTERRUPT_0,
	    ADI_GPIO_PIN_10,
	    true);
	CheckResult(result, "adi_gpio_EnablePinInterruptMask");

	/*
	 * Setup Push Button 2
	 */

	/* set GPIO input */
	result = adi_gpio_SetDirection(
	    ADI_GPIO_PORT_E,
	    ADI_GPIO_PIN_1,
	    ADI_GPIO_DIRECTION_INPUT);
	CheckResult(result, "adi_gpio_SetDirection");

	/* set edge sense mode (PORT E is connected to Pin Interrupt 3) */
	result = adi_gpio_SetPinIntEdgeSense(
		ADI_GPIO_PIN_INTERRUPT_3,
	    ADI_GPIO_PIN_1,
	    ADI_GPIO_SENSE_RISING_EDGE);
	CheckResult(result, "adi_gpio_SetPinIntEdgeSense");

	/* register pinint 3 callback */
	result = adi_gpio_RegisterCallback(
		ADI_GPIO_PIN_INTERRUPT_3,
	    ADI_GPIO_PIN_1,
	    pinIntCallback,
	    (void*)0);
	CheckResult(result, "adi_gpio_RegisterCallback");

	/* set pin interrupt mask */
	result = adi_gpio_EnablePinInterruptMask(
		ADI_GPIO_PIN_INTERRUPT_3,
	    ADI_GPIO_PIN_1,
	    true);
	CheckResult(result, "adi_gpio_EnablePinInterruptMask");

	/*
	 * Setup LED's
	 */

	/* set GPIO output LED 1 */
	result = adi_gpio_SetDirection(
	    ADI_GPIO_PORT_G,
	    ADI_GPIO_PIN_14,
	    ADI_GPIO_DIRECTION_OUTPUT);
	CheckResult(result, "adi_gpio_SetDirection");

	/* set GPIO output LED 2 */
	result = adi_gpio_SetDirection(
	    ADI_GPIO_PORT_G,
	    ADI_GPIO_PIN_15,
	    ADI_GPIO_DIRECTION_OUTPUT);
	CheckResult(result, "adi_gpio_SetDirection");



	/* wait for push button interrupts */
	while (1);



	return 0;
}
